from pydantic import BaseModel


class LanguageModel:
    not_owner_error: str
    not_judge_error: str
    animal_not_found: str
    show_not_found: str
    foreign_keys_error: str
    duplicate_error: str
    show_result_status_error: str
    question_status: str
    no_empty_field: str
    invalid_password: str
    animal_standard_error: str
    out_password: str
    yes: str
    no: str
    role_register_error: str
    show_unregister_status_error: str
    not_registered_error: str
    already_registered_error: str
    show_register_status_error: str
    user_not_found: str
    input_invalid: str
    question_year: str
    question_month: str
    question_day: str
    out_deleted_success: str
    out_deleted_error: str
    out_deleted_animal_active_shows_error: str
    auth_token_expired: str
    get_empty_result: str
    question_name: str
    out_question_name: str
    out_question_breed_id: str
    out_question_animal_id: str
    out_question_show_id: str
    show_start_success: str
    show_stop_success: str
    unregister_animal_success: str
    score_create_success: str
    register_animal_success: str
    out_login: str
    unregister_user_success: str
    register_user_success: str
    show_stop_error_status: str
    show_stop_error_not_all_users_scored: str
    show_start_error_status: str
    show_start_error_no_records_user: str
    show_start_error_no_records_animal: str
    out_question_birth_dt: str
    out_question_user_id: str
    out_question_length: str
    out_question_weight: str
    out_question_height: str
    out_question_sex: str
    out_question_has_defects: str
    out_question_is_multicolor: str
    out_question_is_multi_breed: str
    out_question_show_class: str
    out_question_country: str
    question_breed_id: str
    question_species_id: str
    question_standard_id: str
    out_question_species_id: str
    out_question_standard_id: str
    question_animal_id: str
    question_animalshow_id: str
    question_usershow_id: str
    out_question_animalshow_id: str
    out_question_usershow_id: str
    question_show_id: str
    question_birth_dt: str
    question_user_id: str
    question_length: str
    question_country: str
    question_show_class: str
    question_weight: str
    question_height: str
    question_sex: str
    question_has_defects: str
    question_is_multicolor: str
    question_is_multi_breed: str
    cancel_input: str
    out_id: str
    out_rank: str
    out_animal_id: str
    out_show_id: str
    out_animalshow_id: str
    out_usershow_id: str
    out_breed_id: str
    out_show_name: str
    out_name: str
    out_score_value: str
    out_is_archived: str
    out_dt_created: str
    out_birth_dt: str
    out_user_id: str
    out_length: str
    out_weight: str
    out_height: str
    out_length_unit: str
    out_weight_unit: str
    out_height_unit: str
    out_has_defects: str
    out_is_multicolor: str
    out_is_multi_breed: str
    out_species_id: str
    out_standard_id: str
    out_sex: str
    out_show_class: str
    out_status: str
    out_country: str
    out_sex_female: str
    out_sex_male: str
    out_sex_female_short: str
    out_sex_male_short: str
    question_score_value: str
    out_question_score_value: str
